var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/FormRender/utils/excelUtil.ts
var excelUtil_exports = {};
__export(excelUtil_exports, {
  default: () => excelUtil_default
});
module.exports = __toCommonJS(excelUtil_exports);
var import_exceljs = require("exceljs");
var import_file_saver = require("file-saver");
var import_lodash_es = require("lodash-es");
var import_index = require("./index");
var ExcelUtil = class {
  constructor(schema) {
    this.keyMap = /* @__PURE__ */ new Map();
    this.dateTypes = ["Date", "DateTime"];
    this.dateTrans = {
      Date: "YYYY/MM/DD",
      DateTime: "YYYY/MM/DD HH:mm:ss",
      DateRange: "YYYY/MM/DD-YYYY/MM/DD",
      DateTimeRange: "YYYY/MM/DD HH:mm:ss-YYYY/MM/DD HH:mm:ss"
    };
    this.trans = {
      Date: (value) => this.handleDateValue(value),
      DateTime: (value) => this.handleDateValue(value),
      DateRange: (value) => this.handleDateRangeValue(value),
      DateTimeRange: (value) => this.handleDateRangeValue(value),
      Int: (value) => this.handleNumberValue(value),
      Boolean: (value) => (value == null ? void 0 : value.trim()) === "是",
      Percent: (value) => this.handlePercentValue(value),
      Money: (value) => this.handleNumberValue(value),
      Number: (value) => this.handleNumberValue(value),
      SelectMultiple: (value) => this.handleArrayValue(value),
      Checkbox: (value) => this.handleArrayValue(value),
      String: (value) => this.handleStringValue(value)
    };
    this.init = (schema) => {
      var _a;
      (_a = (0, import_index.filterSchema)(schema)) == null ? void 0 : _a.filter((key) => {
        var _a2;
        return !((_a2 = schema[key]) == null ? void 0 : _a2.hidden);
      }).forEach((key) => {
        var _a2;
        this.keyMap.set(key, (_a2 = schema[key]) == null ? void 0 : _a2.title);
      });
    };
    this.buildHeaderTitle = (title, schemaItem) => {
      let header = title;
      if (schemaItem.required) {
        header = `*${header}`;
      }
      if (this.dateTypes.includes(schemaItem == null ? void 0 : schemaItem.type)) {
        const type = (schemaItem == null ? void 0 : schemaItem.valueType) || (schemaItem == null ? void 0 : schemaItem.type);
        header = `${header} (${this.dateTrans[type]})`;
      }
      return header;
    };
    this.buildValidationMap = (schemaItem) => {
      const type = (schemaItem == null ? void 0 : schemaItem.valueType) || (schemaItem == null ? void 0 : schemaItem.type);
      const validationRules = {
        "Percent,Money,Number": {
          type: "decimal",
          error: "请输入数字"
        },
        Int: {
          type: "whole",
          error: "请输入整数"
        }
      };
      for (const [types, rule] of Object.entries(validationRules)) {
        if (types.split(",").includes(type)) {
          return {
            ...rule,
            showErrorMessage: true,
            errorStyle: "error",
            errorTitle: "错误"
          };
        }
      }
      const dateTrans = this.dateTrans[type];
      if (dateTrans) {
        return {
          type: "date",
          showErrorMessage: true,
          errorStyle: "error",
          errorTitle: "错误",
          error: `请输入正确的日期${dateTrans}`
        };
      }
      return null;
    };
    this.convertIndexToLetter = (index) => {
      let result = "";
      let _index = index;
      while (_index >= 0) {
        result = String.fromCharCode(65 + _index % 26) + result;
        _index = Math.floor(_index / 26) - 1;
      }
      return result;
    };
    this.exportExcelFile = (data, fileName = "example.xlsx", sheetName = "Sheet1", openValidation = false) => {
      const workBook = new import_exceljs.Workbook();
      const sheet = workBook.addWorksheet(sheetName);
      const validationRules = [];
      const headers = [...this.keyMap.entries()].map(([key, value]) => {
        const schemaItem = this.schema[key];
        const header = this.buildHeaderTitle(value, schemaItem);
        if (openValidation) {
          validationRules.push(this.buildValidationMap(schemaItem));
        }
        return {
          header,
          key,
          width: (header == null ? void 0 : header.length) * 3
        };
      });
      sheet.columns = headers;
      if (openValidation) {
        validationRules.forEach((rule, index) => {
          if (rule) {
            const columnKey = this.convertIndexToLetter(index);
            sheet.getCell(`${columnKey}2`).dataValidation = rule;
          }
        });
      }
      const titleRow = sheet.getRow(1);
      titleRow.height = 20;
      titleRow.eachCell(this.applyHeaderStyle);
      sheet.addRows(data);
      workBook.xlsx.writeBuffer().then((buffer) => {
        const file = new Blob([buffer], { type: "application/octet-stream" });
        (0, import_file_saver.saveAs)(file, fileName);
      });
    };
    this.applyHeaderStyle = (cell) => {
      cell.font = {
        name: "黑体",
        bold: true,
        size: 14,
        color: { argb: "333333" }
      };
      cell.alignment = {
        vertical: "middle",
        horizontal: "center"
      };
      cell.fill = {
        type: "pattern",
        pattern: "solid",
        fgColor: { argb: "cccccc" }
      };
    };
    this.validateHeaders = (headerRow, expectedHeaders) => {
      var _a;
      const cleanedHeaders = (_a = headerRow == null ? void 0 : headerRow.filter(Boolean)) == null ? void 0 : _a.map(
        (header) => header == null ? void 0 : header.replace(
          /[ (YYYY/MM/DD)]|[*]|[ (YYYY/MM/DD HH:mm:ss)]|[ (YYYY/MM/DD\-YYYY/MM/DD)]|[ (YYYY/MM/DD HH:mm:ss\-YYYY/MM/DD HH:mm:ss)]/g,
          ""
        )
      );
      return (cleanedHeaders == null ? void 0 : cleanedHeaders.length) === (expectedHeaders == null ? void 0 : expectedHeaders.length) && (cleanedHeaders == null ? void 0 : cleanedHeaders.every(
        (header, index) => header === expectedHeaders[index]
      ));
    };
    this.processRow = (row, header) => {
      return row.reduce((acc, val, index) => {
        var _a, _b, _c, _d;
        if (index < header.length) {
          const key = header[index];
          const type = ((_a = this.schema[key]) == null ? void 0 : _a.valueType) || ((_b = this.schema[key]) == null ? void 0 : _b.type);
          acc[key] = (_d = (_c = this.trans)[type]) == null ? void 0 : _d.call(
            _c,
            (0, import_lodash_es.isObject)(val) ? (0, import_lodash_es.isDate)(val) ? val : val == null ? void 0 : val.result : val
          );
        }
        return acc;
      }, {});
    };
    this.readExcelFile = async (file) => {
      var _a;
      try {
        const header = [...this.keyMap.keys()];
        const workBook = new import_exceljs.Workbook();
        await workBook.xlsx.load(file);
        const sheet = workBook.getWorksheet(1);
        if (!sheet)
          throw new Error("无法读取文件");
        const rows = ((_a = sheet.getSheetValues()) == null ? void 0 : _a.filter((row) => row == null ? void 0 : row.length)) || [];
        if (rows.length < 2)
          throw new Error("文件内容为空");
        const headerRow = rows[0];
        const expectedHeaders = header.map((key) => {
          var _a2;
          return (_a2 = this.schema[key]) == null ? void 0 : _a2.title;
        });
        if (!this.validateHeaders(headerRow, expectedHeaders)) {
          throw new Error("模板与表头不一致");
        }
        return rows.slice(1).map((row) => this.processRow(row.slice(1), header));
      } catch (error) {
        throw error instanceof Error ? error : new Error("文件处理失败");
      }
    };
    this.schema = schema;
    this.init(schema);
  }
  handleDateValue(value) {
    return value && (0, import_lodash_es.isDate)(new Date(value)) ? value : null;
  }
  handleDateRangeValue(value) {
    var _a;
    if (!value)
      return [];
    const valueArr = (_a = value.split("-")) == null ? void 0 : _a.map((item) => item == null ? void 0 : item.trim());
    return valueArr.some((item) => !(0, import_lodash_es.isDate)(new Date(item))) ? [] : valueArr;
  }
  handleNumberValue(value) {
    if ((0, import_lodash_es.isUndefined)(value))
      return null;
    const num = (0, import_lodash_es.toNumber)(value);
    return (0, import_lodash_es.isNumber)(num) ? num : null;
  }
  handlePercentValue(value) {
    var _a;
    if (!value)
      return null;
    const stringValue = (_a = (0, import_lodash_es.toString)(value)) == null ? void 0 : _a.trim();
    return (stringValue == null ? void 0 : stringValue.includes(".")) ? (0, import_lodash_es.toString)((0, import_lodash_es.toNumber)(stringValue) * 100) : stringValue;
  }
  handleArrayValue(value) {
    return value ? value.split(",").map((item) => item == null ? void 0 : item.trim()) : null;
  }
  handleStringValue(value) {
    var _a;
    if ((0, import_lodash_es.isNumber)(value)) {
      return (_a = (0, import_lodash_es.toString)(value)) == null ? void 0 : _a.trim();
    }
    return value == null ? void 0 : value.trim();
  }
};
var excelUtil_default = ExcelUtil;
